@@ -19,14 +19,8 @@
  * USA
  */
 
-#include <goffice/graph/gog-chart-map.h>
-#include <goffice/math/go-cspline.h>
-#include <goffice/math/go-math.h>
-#include <goffice/math/go-rangefunc.h>
-#include <goffice/utils/go-bezier.h>
-#include <goffice/utils/go-line.h>
-#include <goffice/utils/go-path.h>
-#include <goffice/utils/go-geometry.h>
+#include <goffice/goffice-config.h>
+#include <goffice/goffice.h>
 
 struct _GogChartMap {
 	GogChart 		*chart;
@@ -36,6 +30,7 @@ struct _GogChartMap {
 	gboolean		 is_valid;
 
 	void 	 (*map_2D_to_view) 	(GogChartMap *map, double x, double y, double *u, double *v);
+	void 	 (*map_view_to_2D) 	(GogChartMap *map, double x, double y, double *u, double *v);
 	double 	 (*map_2D_derivative_to_view) (GogChartMap *map, double deriv, double x, double y);
 	GOPath  *(*make_path)	   	(GogChartMap *map, double const *x, double const *y, int n_points,
 					 GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data);
@@ -173,6 +168,15 @@ x_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
 	*v = data->a * y + data->b;
 }
 
+static void
+x_map_view_to_2D (GogChartMap *map, double x, double y, double *u, double *v)
+{
+	XMapData *data = map->data;
+
+	*u = gog_axis_map_from_view (map->axis_map[0], x);
+	*v =  (y - data->b) / data->a;
+}
+
 typedef struct {
 	double		a[2][2];
 	double		b[2];
@@ -197,6 +201,13 @@ xy_map_2D_derivative_to_view (GogChartMap *map, double deriv, double x, double y
 	return (isnan (d))? go_nan: deriv * d;
 }
 
+static void
+xy_map_view_to_2D (GogChartMap *map, double x, double y, double *u, double *v)
+{
+	*u = gog_axis_map_from_view (map->axis_map[0], x);
+	*v = gog_axis_map_from_view (map->axis_map[1], y);
+}
+
 static GOPath *
 make_path_linear (GogChartMap *map,
 		  double const *x, double const *y,
@@ -255,8 +266,7 @@ make_path_spline (GogChartMap *map,
 	double u, v;
 	double yy, yy_min, yy_max;
 	gboolean is_inverted;
-	//struct GOCSpline *splinex, *spliney;
-	struct GOBezierSpline *spline;
+	GOBezierSpline *spline;
 
 	path = go_path_new ();
 	if (n_points < 1)
@@ -329,7 +339,7 @@ make_path_cspline (GogChartMap *map,
 	int i, n_valid_points = 0;
 	double *uu, *vv, u, v;
 	double p0 = 0., p1 = 0.; /* clamped derivatives */
-	struct GOCSpline *spline;
+	GOCSpline *spline;
 
 	path = go_path_new ();
 	if (n_points < 1 || !go_range_increasing (x, n_points))
@@ -428,19 +438,19 @@ xy_make_path_step (GogChartMap *map, double const *x, double const *y, int n_poi
 
 	n_valid_points = 0;
 	for (i = 0; i < n_points; i++) {
-		gog_chart_map_2D_to_view (map, 
-					  x != NULL ? x[i] : i + 1, 
+		gog_chart_map_2D_to_view (map,
+					  x != NULL ? x[i] : i + 1,
 					  y != NULL ? y[i] : i + 1, &xx, &yy);
-		if (go_finite (xx) 
+		if (go_finite (xx)
 		    && go_finite (yy)
 		    && fabs (xx) != DBL_MAX
 		    && fabs (yy) != DBL_MAX) {
 			n_valid_points++;
 
-			if (n_valid_points == 1) 
+			if (n_valid_points == 1)
 				go_path_move_to (path, xx, yy);
 			else
-				switch (interpolation) {	       
+				switch (interpolation) {
 					case GO_LINE_INTERPOLATION_STEP_START:
 						go_path_line_to (path, xx, last_yy);
 						break;
@@ -472,7 +482,7 @@ xy_make_path_step (GogChartMap *map, double const *x, double const *y, int n_poi
 }
 
 static GOPath *
-xy_make_path (GogChartMap *map, double const *x, double const *y, 
+xy_make_path (GogChartMap *map, double const *x, double const *y,
 	      int n_points, GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data)
 {
 	GOPath *path = NULL;
@@ -513,7 +523,7 @@ xy_make_path (GogChartMap *map, double const *x, double const *y,
 }
 
 static GOPath *
-xy_make_close_path (GogChartMap *map, double const *x, double const *y, 
+xy_make_close_path (GogChartMap *map, double const *x, double const *y,
 		    int n_points, GogSeriesFillType fill_type)
 {
 	GogAxisMap *x_map, *y_map;
@@ -589,6 +599,31 @@ polar_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v
 	*v = data->cy + r * data->ry * sin (t);
 }
 
+static void
+polar_map_view_to_2D (GogChartMap *map, double x, double y, double *u, double *v)
+{
+	GogChartMapPolarData *data = (GogChartMapPolarData *) map->data;
+	double r, t;
+
+	x = (x - data->cx) / data->rx;
+	y = (y - data->cy) / data->ry;
+	r = hypot (x, y);
+	t = atan2 (y, x);
+	/* FIXME: following code looks like a kludge */
+	if (gog_axis_map_is_discrete (map->axis_map[0])) {
+		*u = gog_axis_map_from_view (map->axis_map[0], t);
+		if (*u < data->th0)
+			*u = data->th1;
+		else if (*u > data->th1)
+			*u = data->th0;
+	} else {
+		if (t > 0)
+			t -= 2 * M_PI; /* Hmm, why? */
+		*u = gog_axis_map_from_view (map->axis_map[0], t);
+	}
+	*v = gog_axis_map_from_view (map->axis_map[1], r);
+}
+
 static GOPath *
 polar_make_path_step (GogChartMap *map, double const *x, double const *y, int n_points,
 		      GOLineInterpolation interpolation, gboolean skip_invalid)
@@ -778,10 +813,10 @@ gog_chart_map_get_polar_parms (GogChartMap *map)
  * @axis2: 3rd dimension axis
  * @fill_area: does chart fill allocated area
  *
- * Creates a new #GogChartMap, used for conversion from data space 
+ * Creates a new #GogChartMap, used for conversion from data space
  * to canvas space.
  *
- * returns: a new #GogChart object. 
+ * returns: a new #GogChartMap object.
  **/
 
 GogChartMap *
@@ -817,6 +852,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 				data->a = - area->h;
 
 				map->map_2D_to_view = x_map_2D_to_view;
+				map->map_view_to_2D = x_map_view_to_2D;
 				map->map_2D_derivative_to_view = NULL;
 				map->make_path = NULL;
 				map->make_close_path = NULL;
@@ -836,6 +872,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 				map->data = NULL;
 				map->map_2D_to_view = xy_map_2D_to_view;
 				map->map_2D_derivative_to_view = xy_map_2D_derivative_to_view;
+				map->map_view_to_2D = xy_map_view_to_2D;
 				map->make_path = xy_make_path;
 				map->make_close_path = xy_make_close_path;
 
@@ -876,6 +913,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 				map->data = data;
 				map->map_2D_to_view = polar_map_2D_to_view;
 				map->map_2D_derivative_to_view = NULL;
+				map->map_view_to_2D = polar_map_view_to_2D;
 				map->make_path = polar_make_path;
 				map->make_close_path = polar_make_close_path;
 
@@ -887,6 +925,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 			g_warning ("[GogChartMap::new] unimplemented for axis set %d", axis_set);
 			map->map_2D_to_view = null_map_2D;
 			map->map_2D_derivative_to_view = NULL;
+			map->map_view_to_2D = null_map_2D;
 			break;
 	}
 
@@ -910,7 +949,6 @@ gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, doubl
 	(map->map_2D_to_view) (map, x, y, u, v);
 }
 
-
 /**
  * gog_chart_map_2D_derivative_to_view:
  * @map: a #GogChartMap
@@ -930,6 +968,23 @@ gog_chart_map_2D_derivative_to_view (GogChartMap *map, double deriv, double x, d
 		map->map_2D_derivative_to_view (map, deriv, x, y): go_nan;
 }
 
+/**
+ * gog_chart_map_view_to_2D:
+ * @map: a #GogChartMap
+ * @x: data x value
+ * @y: data y value
+ * @u: placeholder for x converted value
+ * @v: placeholder for y converted value
+ *
+ * Converts a 2D coordinate from canvas space to data space.
+ **/
+
+void
+gog_chart_map_view_to_2D (GogChartMap *map, double x, double y, double *u, double *v)
+{
+	(map->map_view_to_2D) (map, x, y, u, v);
+}
+
 /**
  * gog_chart_map_get_axis_map:
  * @map: a #GogChartMap
@@ -958,8 +1013,8 @@ gog_chart_map_get_axis_map (GogChartMap *map, unsigned int i)
  * Tests if @map was correctly initializied, i.e. if all associated axis_map
  * are valid (see gog_axis_map_is_valid() ).
  *
- * given 
- * to gog_chart_map_new. 
+ * given
+ * to gog_chart_map_new.
  * returns: %TRUE if @map is valid.
  **/
 
