@@ -10,6 +10,7 @@
  */
 #include <goffice/goffice-config.h>
 #include <goffice/goffice.h>
+#include <goffice/gtk/go-gtk-compat.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
@@ -20,7 +21,7 @@
 
 struct _GOFontSel {
 	GtkHBox		box;
-	GladeXML	*gui;
+	GtkBuilder	*gui;
 
 	GtkWidget	*font_name_entry;
 	GtkWidget	*font_style_entry;
@@ -88,10 +89,14 @@ cb_list_adjust (GtkTreeView* view)
 	int pos, height, child_height;
 
 	if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (view), &model, &iter)) {
+		GtkAllocation allocation;
+		GtkRequisition requisition;
+		gtk_widget_get_allocation (GTK_WIDGET (view), &allocation);
+		gtk_widget_get_child_requisition (GTK_WIDGET (view), &requisition);
 		path = gtk_tree_model_get_path (model, &iter);
 		scroll = GTK_SCROLLED_WINDOW (gtk_widget_get_parent (GTK_WIDGET (view)));
-		height = GTK_WIDGET (view)->allocation.height;
-		child_height = GTK_WIDGET (view)->requisition.height;
+		height = allocation.height;
+		child_height = requisition.height;
 		if (height < child_height) {
 			gtk_tree_view_get_cell_area (view, path, NULL, &rect);
 			adj = gtk_scrolled_window_get_vadjustment (scroll);
@@ -225,7 +230,7 @@ select_row (GtkTreeView *list, int row)
 		path = gtk_tree_path_new_from_indices (row, -1);
 
 		gtk_tree_selection_select_path (gtk_tree_view_get_selection (list), path);
-		if (GTK_WIDGET_REALIZED (list))
+		if (gtk_widget_is_realized (list))
 			cb_list_adjust (list);
 		gtk_tree_path_free (path);
 	}
@@ -334,26 +339,26 @@ gfs_init (GOFontSel *gfs)
 {
 	GtkWidget *w;
 
-	gfs->gui = go_glade_new ("go-font-sel.glade", "toplevel-table", GETTEXT_PACKAGE, NULL);
+	gfs->gui = go_gtk_builder_new ("go-font-sel.ui", GETTEXT_PACKAGE, NULL);
 	if (gfs->gui == NULL)
                 return;
 
 	gfs->modifications = pango_attr_list_new ();
 
-	gtk_box_pack_start (GTK_BOX (gfs), 
-		glade_xml_get_widget (gfs->gui, "toplevel-table"), 
-		TRUE, TRUE, 0);
-	gfs->font_name_entry  = glade_xml_get_widget (gfs->gui, "font-name-entry");
-	gfs->font_style_entry = glade_xml_get_widget (gfs->gui, "font-style-entry");
-	gfs->font_size_entry  = glade_xml_get_widget (gfs->gui, "font-size-entry");
-	gfs->font_name_list  = GTK_TREE_VIEW (glade_xml_get_widget (gfs->gui, "font-name-list"));
-	gfs->font_style_list = GTK_TREE_VIEW (glade_xml_get_widget (gfs->gui, "font-style-list"));
-	gfs->font_size_list  = GTK_TREE_VIEW (glade_xml_get_widget (gfs->gui, "font-size-list"));
+	gtk_box_pack_start (GTK_BOX (gfs),
+		go_gtk_builder_get_widget (gfs->gui, "toplevel-table"),
+  		TRUE, TRUE, 0);
+	gfs->font_name_entry  = go_gtk_builder_get_widget (gfs->gui, "font-name-entry");
+	gfs->font_style_entry = go_gtk_builder_get_widget (gfs->gui, "font-style-entry");
+	gfs->font_size_entry  = go_gtk_builder_get_widget (gfs->gui, "font-size-entry");
+	gfs->font_name_list  = GTK_TREE_VIEW (gtk_builder_get_object (gfs->gui, "font-name-list"));
+	gfs->font_style_list = GTK_TREE_VIEW (gtk_builder_get_object (gfs->gui, "font-style-list"));
+	gfs->font_size_list  = GTK_TREE_VIEW (gtk_builder_get_object (gfs->gui, "font-size-list"));
 
 	w = GTK_WIDGET (g_object_new (GOC_TYPE_CANVAS, NULL));
 	gfs->font_preview_canvas = GOC_CANVAS (w);
 	gtk_widget_show_all (w);
-	w = glade_xml_get_widget (gfs->gui, "font-preview-frame");
+	w = go_gtk_builder_get_widget (gfs->gui, "font-preview-frame");
 	gtk_container_add (GTK_CONTAINER (w), GTK_WIDGET (gfs->font_preview_canvas));
 
 	gfs->font_preview_text = goc_item_new (
@@ -493,7 +498,7 @@ go_font_sel_set_style (GOFontSel *gfs, gboolean is_bold, gboolean is_italic)
 	}
 	select_row (gfs->font_style_list, n);
 
-	go_font_sel_add_attr (gfs, 
+	go_font_sel_add_attr (gfs,
 		pango_attr_weight_new (is_bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL),
 		pango_attr_style_new (is_italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL));
 /* FIXME FIXME FIXME Do we really need the following line? */
