@@ -18,14 +18,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
-#include <goffice/goffice-config.h>
-#include "goffice-gtk.h"
 
-#include <goffice/app/go-cmd-context.h>
-#include <goffice/utils/go-file.h>
+#include <goffice/goffice-config.h>
+#include <goffice/goffice.h>
 #include <goffice/goffice-priv.h>
-#include <goffice/gtk/go-pixbuf.h>
-#include <gtk/gtk.h>
+#include <goffice/gtk/go-gtk-compat.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <atk/atkrelation.h>
 #include <atk/atkrelationset.h>
@@ -47,7 +45,7 @@
 
 
 /**
- * go_gtk_button_new_with_stock
+ * go_gtk_button_new_with_stock:
  * @text : button label
  * @stock_id : id for stock icon
  *
@@ -76,7 +74,7 @@ go_gtk_button_new_with_stock (char const *text, char const* stock_id)
  *
  * FROM : gedit
  * Creates and adds a button with stock image to the action area of an existing dialog.
- * 
+ *
  * Returns: newly created button
  **/
 GtkWidget*
@@ -92,109 +90,19 @@ go_gtk_dialog_add_button (GtkDialog *dialog, char const* text, char const* stock
 	button = go_gtk_button_new_with_stock (text, stock_id);
 	g_return_val_if_fail (button != NULL, NULL);
 
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default (button, TRUE);
 
 	gtk_widget_show (button);
 
-	gtk_dialog_add_action_widget (dialog, button, response_id);	
+	gtk_dialog_add_action_widget (dialog, button, response_id);
 
 	return button;
 }
 
 /**
- * go_glade_new :
- * @gladefile : the name of the file load
- * @root :
- * @domain :
- * @gcc : #GOCmdContext
- *
- * Simple utility to open glade files
- *
- * 0.4.3 : renamed from go_libglade_new
- *
- * Returns: a new #GladeXML or NULL
- **/
-GladeXML *
-go_glade_new (char const *gladefile, char const *root,
-	      char const *domain, GOCmdContext *gcc)
-{
-	GladeXML *gui;
-	char *f;
-
-	g_return_val_if_fail (gladefile != NULL, NULL);
-
-	if (!g_path_is_absolute (gladefile))
-		f = g_build_filename (go_sys_data_dir (), "glade", gladefile, NULL);
-	else
-		f = g_strdup (gladefile);
-
-	gui = glade_xml_new (f, root, domain);
-	if (gui == NULL && gcc != NULL) {
-		char *msg = g_strdup_printf (_("Unable to open file '%s'"), f);
-		go_cmd_context_error_system (gcc, msg);
-		g_free (msg);
-	}
-	g_free (f);
-
-	return gui;
-}
-
-/**
- * go_glade_signal_connect :
- * @gui : #GladeXML
- * @instance_name : widget name
- * @detailed_signal : signal name
- * @c_handler : #GCallback
- * @data : arbitrary
- *
- * Convenience wrapper around g_signal_connect for glade.
- *
- * Returns: The signal id
- **/
-gulong
-go_glade_signal_connect	(GladeXML	*gui,
-			 gchar const	*instance_name,
-			 gchar const	*detailed_signal,
-			 GCallback	 c_handler,
-			 gpointer	 data)
-{
-	GtkWidget *w;
-	g_return_val_if_fail (gui != NULL, 0);
-	w = glade_xml_get_widget (gui, instance_name);
-	g_return_val_if_fail (w != NULL, 0);
-	return g_signal_connect (w, detailed_signal, c_handler, data);
-}
-
-/**
- * go_glade_signal_connect_swapped :
- * @gui : #GladeXML
- * @instance_name : widget name
- * @detailed_signal : signal name
- * @c_handler : #GCallback
- * @data : arbitary
- *
- * Convenience wrapper around g_signal_connect_swapped for glade.
- *
- * Returns: The signal id
- **/
-gulong
-go_glade_signal_connect_swapped (GladeXML	*gui,
-				 gchar const	*instance_name,
-				 gchar const	*detailed_signal,
-				 GCallback	 c_handler,
-				 gpointer	 data)
-{
-	GtkWidget *w;
-	g_return_val_if_fail (gui != NULL, 0);
-	w = glade_xml_get_widget (gui, instance_name);
-	g_return_val_if_fail (w != NULL, 0);
-	return g_signal_connect_swapped (w, detailed_signal, c_handler, data);
-}
-
-/**
- * go_xml_builder_new :
+ * go_gtk_builder_new :
  * @uifile : the name of the file load
- * @domain :
+ * @domain : the translation domain
  * @gcc : #GOCmdContext
  *
  * Simple utility to open ui files
@@ -202,7 +110,7 @@ go_glade_signal_connect_swapped (GladeXML	*gui,
  * Returns: a new #GtkBuilder or NULL
  **/
 GtkBuilder *
-go_xml_builder_new (char const *uifile,
+go_gtk_builder_new (char const *uifile,
 		    char const *domain, GOCmdContext *gcc)
 {
 	GtkBuilder *gui;
@@ -212,13 +120,17 @@ go_xml_builder_new (char const *uifile,
 	g_return_val_if_fail (uifile != NULL, NULL);
 
 	if (!g_path_is_absolute (uifile))
-		f = g_build_filename (go_sys_data_dir (), "glade", uifile, NULL);
+		f = g_build_filename (go_sys_data_dir (), "ui", uifile, NULL);
 	else
 		f = g_strdup (uifile);
 
 	gui = gtk_builder_new ();
-	gtk_builder_set_translation_domain (gui, domain);
-	gtk_builder_add_from_file (gui, f, &error);
+	if (domain)
+		gtk_builder_set_translation_domain (gui, domain);
+	if (!gtk_builder_add_from_file (gui, f, &error)) {
+		g_object_unref (gui);
+		gui = NULL;
+	}
 	if (gui == NULL && gcc != NULL) {
 		char *msg;
 		if (error) {
@@ -234,6 +146,122 @@ go_xml_builder_new (char const *uifile,
 	return gui;
 }
 
+/**
+ * go_gtk_builder_signal_connect :
+ * @gui : #GtkBuilder
+ * @instance_name : widget name
+ * @detailed_signal : signal name
+ * @c_handler : #GCallback
+ * @data : arbitrary
+ *
+ * Convenience wrapper around g_signal_connect for GtkBuilder.
+ *
+ * Returns: The signal id
+ **/
+gulong
+go_gtk_builder_signal_connect	(GtkBuilder	*gui,
+			 gchar const	*instance_name,
+			 gchar const	*detailed_signal,
+			 GCallback	 c_handler,
+			 gpointer	 data)
+{
+	GObject *obj;
+	g_return_val_if_fail (gui != NULL, 0);
+	obj = gtk_builder_get_object (gui, instance_name);
+	g_return_val_if_fail (obj != NULL, 0);
+	return g_signal_connect (obj, detailed_signal, c_handler, data);
+}
+
+/**
+ * go_xml_builder_signal_connect_swapped :
+ * @gui : #GtkBuilder
+ * @instance_name : widget name
+ * @detailed_signal : signal name
+ * @c_handler : #GCallback
+ * @data : arbitary
+ *
+ * Convenience wrapper around g_signal_connect_swapped for GtkBuilder.
+ *
+ * Returns: The signal id
+ **/
+gulong
+go_gtk_builder_signal_connect_swapped (GtkBuilder	*gui,
+				 gchar const	*instance_name,
+				 gchar const	*detailed_signal,
+				 GCallback	 c_handler,
+				 gpointer	 data)
+{
+	GObject *obj;
+	g_return_val_if_fail (gui != NULL, 0);
+	obj = gtk_builder_get_object (gui, instance_name);
+	g_return_val_if_fail (obj != NULL, 0);
+	return g_signal_connect_swapped (obj, detailed_signal, c_handler, data);
+}
+
+/**
+ * go_gtk_builder_get_widget :
+ * @gui : the #GtkBuilder
+ * @widget_name : the name of the combo box in the ui file.
+ *
+ * Simple wrapper to #gtk_builder_get_object which returns the object
+ * as a GtkWidget.
+ *
+ * Returns: a new #GtkWidget or NULL
+ **/
+GtkWidget *
+go_gtk_builder_get_widget (GtkBuilder *gui, char const *widget_name)
+{
+	return GTK_WIDGET (gtk_builder_get_object (gui, widget_name));
+}
+
+/**
+ * go_gtk_builder_combo_box_init_text :
+ * @gui : the #GtkBuilder
+ * @widget_name : the name of the combo box in the ui file.
+ *
+ * searches the #GtkComboBox in @gui and ensures it has a model and a
+ * renderer appropriate for using with #gtk_combo_box_append_text and friends.
+ *
+ * Returns: the #GtkComboBox or NULL
+ **/
+GtkComboBox *
+go_gtk_builder_combo_box_init_text (GtkBuilder *gui, char const *widget_name)
+{
+	GtkComboBox *box;
+	GList *cells;
+	g_return_val_if_fail (gui != NULL, NULL);
+	box = GTK_COMBO_BOX (gtk_builder_get_object (gui, widget_name));
+	/* search for the model and create one if none exists */
+	g_return_val_if_fail (box != NULL, NULL);
+	if (gtk_combo_box_get_model (box) == NULL) {
+		GtkListStore *store = gtk_list_store_new (1, G_TYPE_STRING);
+		gtk_combo_box_set_model (box, GTK_TREE_MODEL (store));
+		g_object_unref (store);
+	}
+	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (box));
+	if (g_list_length (cells) == 0) {
+		/* populate the cell layout */
+		GtkCellRenderer *cell = gtk_cell_renderer_text_new ();
+		gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (box), cell, TRUE);
+		gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (box), cell,
+						"text", 0, NULL);
+	}
+	g_list_free (cells);
+	return box;
+}
+
+int
+go_gtk_builder_group_value (GtkBuilder *gui, char const * const group[])
+{
+	int i;
+	for (i = 0; group[i]; i++) {
+		GtkWidget *w = go_gtk_builder_get_widget (gui, group[i]);
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+			return i;
+	}
+	return -1;
+}
+
 /*
  * A variant of gtk_window_activate_default that does not end up reactivating
  * the widget that [Enter] was pressed in.
@@ -241,8 +269,9 @@ go_xml_builder_new (char const *uifile,
 static void
 cb_activate_default (GtkWindow *window)
 {
-	if (window->default_widget && GTK_WIDGET_IS_SENSITIVE (window->default_widget))
-		gtk_widget_activate (window->default_widget);
+	GtkWidget *w = gtk_window_get_default_widget (window);
+	if (w && gtk_widget_is_sensitive (w))
+		gtk_widget_activate (w);
 }
 
 /**
@@ -277,7 +306,7 @@ go_gtk_widget_disable_focus (GtkWidget *w)
 	if (GTK_IS_CONTAINER (w))
 		gtk_container_foreach (GTK_CONTAINER (w),
 			(GtkCallback) go_gtk_widget_disable_focus, NULL);
-	GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (w, FALSE);
 }
 
 /**
@@ -308,7 +337,7 @@ go_pango_measure_string (PangoContext *context, PangoFontDescription const *font
 static void
 cb_parent_mapped (GtkWidget *parent, GtkWindow *window)
 {
-	if (GTK_WIDGET_MAPPED (window)) {
+	if (gtk_widget_is_mapped (window)) {
 		gtk_window_present (window);
 		g_signal_handlers_disconnect_by_func (G_OBJECT (parent),
 			G_CALLBACK (cb_parent_mapped), window);
@@ -343,7 +372,7 @@ go_gtk_window_set_transient (GtkWindow *toplevel, GtkWindow *window)
 
 	gtk_window_set_position (window, position);
 
-	if (!GTK_WIDGET_MAPPED (toplevel))
+	if (!gtk_widget_is_mapped (toplevel))
 		g_signal_connect_after (G_OBJECT (toplevel),
 			"map",
 			G_CALLBACK (cb_parent_mapped), window);
@@ -627,7 +656,7 @@ typedef struct {
 } SaveInfoState;
 
 static void
-save_info_state_free (SaveInfoState *state) 
+save_info_state_free (SaveInfoState *state)
 {
 	g_free (state->uri);
 	g_free (state);
@@ -639,8 +668,8 @@ cb_format_combo_changed (GtkComboBox *combo, GtkWidget *expander)
 	GOImageFormatInfo const *format_info;
 
 	format_info = go_image_get_format_info (gtk_combo_box_get_active (combo));
-	gtk_widget_set_sensitive (expander, 
-				  format_info != NULL && 
+	gtk_widget_set_sensitive (expander,
+				  format_info != NULL &&
 				  format_info->is_dpi_useful);
 }
 
@@ -668,7 +697,7 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 	GtkWidget *expander = NULL;
 	GtkWidget *resolution_spin = NULL;
 	GtkWidget *resolution_table;
-	GladeXML *gui;
+	GtkBuilder *gui;
 	SaveInfoState *state;
 	char const *key = "go_gui_get_image_save_info";
 	char *uri = NULL;
@@ -687,8 +716,7 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 
 	g_object_set (G_OBJECT (fsel), "title", _("Save as"), NULL);
 
-	gui = go_glade_new ("go-image-save-dialog-extra.glade", 
-			       "image_save_dialog_extra", 
+	gui = go_gtk_builder_new ("go-image-save-dialog-extra.ui",
 			       GETTEXT_PACKAGE, NULL);
 	if (gui != NULL) {
 		GtkWidget *widget;
@@ -697,40 +725,40 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 		if (supported_formats != NULL && ret_format != NULL) {
 			int i;
 			GSList *l;
-			format_combo = GTK_COMBO_BOX (glade_xml_get_widget (gui, "format_combo"));
+			format_combo = go_gtk_builder_combo_box_init_text (gui, "format_combo");
 			for (l = supported_formats, i = 0; l != NULL; l = l->next, i++) {
 				format = GPOINTER_TO_UINT (l->data);
 				format_info = go_image_get_format_info (format);
-				gtk_combo_box_append_text (format_combo, _(format_info->desc)); 
-				if (format == state->format) 
+				gtk_combo_box_append_text (format_combo, _(format_info->desc));
+				if (format == state->format)
 					gtk_combo_box_set_active (format_combo, i);
 			}
 			if (gtk_combo_box_get_active (format_combo) < 0)
 				gtk_combo_box_set_active (format_combo, 0);
 		} else {
-			widget = glade_xml_get_widget (gui, "file_type_box");
+			widget = go_gtk_builder_get_widget (gui, "file_type_box");
 			gtk_widget_hide (widget);
 		}
 
 		if (supported_formats != NULL && ret_format != NULL) {
-			widget = glade_xml_get_widget (gui, "image_save_dialog_extra");
+			widget = go_gtk_builder_get_widget (gui, "image_save_dialog_extra");
 			gtk_file_chooser_set_extra_widget (fsel, widget);
 		}
 
 		/* Export setting expander */
-		expander = glade_xml_get_widget (gui, "export_expander");
+		expander = go_gtk_builder_get_widget (gui, "export_expander");
 		if (resolution != NULL) {
 			gtk_expander_set_expanded (GTK_EXPANDER (expander), state->is_expanded);
-			resolution_spin = glade_xml_get_widget (gui, "resolution_spin");
+			resolution_spin = go_gtk_builder_get_widget (gui, "resolution_spin");
 			gtk_spin_button_set_value (GTK_SPIN_BUTTON (resolution_spin), state->resolution);
-		} else 
+		} else
 			gtk_widget_hide (expander);
 
 		if (resolution != NULL && supported_formats != NULL && ret_format != NULL) {
-			resolution_table = glade_xml_get_widget (gui, "resolution_table");
-		
-			cb_format_combo_changed (format_combo, resolution_table);	
-			g_signal_connect (GTK_WIDGET (format_combo), "changed", 
+			resolution_table = go_gtk_builder_get_widget (gui, "resolution_table");
+
+			cb_format_combo_changed (format_combo, resolution_table);
+			g_signal_connect (GTK_WIDGET (format_combo), "changed",
 					  G_CALLBACK (cb_format_combo_changed), resolution_table);
 		}
 
@@ -741,7 +769,7 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 		gtk_file_chooser_set_uri (fsel, state->uri);
 		gtk_file_chooser_unselect_all (fsel);
 	}
-	
+
 	/* Show file selector */
  loop:
 	if (!go_gtk_file_sel_dialog (toplevel, GTK_WIDGET (fsel)))
@@ -750,8 +778,8 @@ go_gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
 	if (format_combo) {
 		char *new_uri = NULL;
 
-		format = GPOINTER_TO_UINT (g_slist_nth_data 
-			(supported_formats, 
+		format = GPOINTER_TO_UINT (g_slist_nth_data
+			(supported_formats,
 			 gtk_combo_box_get_active (format_combo)));
 		format_info = go_image_get_format_info (format);
 		if (!go_url_check_extension (uri, format_info->ext, &new_uri) &&
@@ -983,24 +1011,19 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 	} else if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		char *dirname = go_dirname_from_uri (uri, TRUE);
 		char *basename = go_basename_from_uri (uri);
-		char *msg = g_markup_printf_escaped (
-			_("A file called <i>%s</i> already exists in %s.\n\n"
-			  "Do you want to save over it?"),
-			basename, dirname);
-		/* gcc complains over this, but the format seems right and
-		   is not supposed to be constant.  Using "%s" is wrong.  */
 		GtkWidget *dialog = gtk_message_dialog_new_with_markup (parent,
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
 			 GTK_MESSAGE_WARNING,
 			 GTK_BUTTONS_OK_CANCEL,
-			 msg);
+			 _("A file called <i>%s</i> already exists in %s.\n\n"
+			   "Do you want to save over it?"),
+			 basename, dirname);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 			overwrite_by_default ? GTK_RESPONSE_OK : GTK_RESPONSE_CANCEL);
 		result = GTK_RESPONSE_OK ==
 			go_gtk_dialog_run (GTK_DIALOG (dialog), parent);
 		g_free (dirname);
 		g_free (basename);
-		g_free (msg);
 	}
 
 	g_free (filename);
@@ -1008,7 +1031,7 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 }
 
 /**
- * go_gtk_dialog_run
+ * go_gtk_dialog_run:
  * @dialog : #GtkDialog
  * @parent : #GtkWindow
  *
@@ -1059,10 +1082,9 @@ go_gtk_notice_dialog (GtkWindow *parent, GtkMessageType type,
 	GtkWidget *dialog;
 
 	VPRINTF_MESSAGE (format, args, msg);
-	dialog = gtk_message_dialog_new (parent,
+	dialog = gtk_message_dialog_new_with_markup (parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT, type, GTK_BUTTONS_OK, "%s", msg);
 	g_free (msg);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
 	go_gtk_dialog_run (GTK_DIALOG (dialog), parent);
 }
 
@@ -1195,7 +1217,7 @@ go_dialog_guess_alternative_button_order (GtkDialog *dialog)
 	gboolean any = FALSE;
 	int loops = 0;
 
-	children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
+	children = gtk_container_get_children (GTK_CONTAINER (gtk_dialog_get_action_area (dialog)));
 	if (!children)
 		return;
 
@@ -1273,13 +1295,20 @@ go_menu_position_below (GtkMenu  *menu,
 	GdkScreen *screen;
 	gint monitor_num;
 	GdkRectangle monitor;
+	GdkWindow *window = gtk_widget_get_window (widget);
+	GtkAllocation size;
 
-	gdk_window_get_origin (widget->window, &sx, &sy);
+	gtk_widget_get_allocation (widget, &size);
 
-	if (GTK_WIDGET_NO_WINDOW (widget))
+	if (window)
+		gdk_window_get_origin (window, &sx, &sy);
+	else
+		sx = sy = 0;
+
+	if (!gtk_widget_get_has_window (widget))
 	{
-		sx += widget->allocation.x;
-		sy += widget->allocation.y;
+		sx += size.x;
+		sy += size.y;
 	}
 
 	gtk_widget_size_request (GTK_WIDGET (menu), &req);
@@ -1287,11 +1316,11 @@ go_menu_position_below (GtkMenu  *menu,
 	if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
 		*x = sx;
 	else
-		*x = sx + widget->allocation.width - req.width;
+		*x = sx + size.width - req.width;
 	*y = sy;
 
 	screen = gtk_widget_get_screen (widget);
-	monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
 	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
 	if (*x < monitor.x)
@@ -1299,12 +1328,12 @@ go_menu_position_below (GtkMenu  *menu,
 	else if (*x + req.width > monitor.x + monitor.width)
 		*x = monitor.x + monitor.width - req.width;
 
-	if (monitor.y + monitor.height - *y - widget->allocation.height >= req.height)
-		*y += widget->allocation.height;
+	if (monitor.y + monitor.height - *y - size.height >= req.height)
+		*y +=size.height;
 	else if (*y - monitor.y >= req.height)
 		*y -= req.height;
-	else if (monitor.y + monitor.height - *y - widget->allocation.height > *y - monitor.y)
-		*y += widget->allocation.height;
+	else if (monitor.y + monitor.height - *y - size.height > *y - monitor.y)
+		*y += size.height;
 	else
 		*y -= req.height;
 
