@@ -172,7 +172,7 @@ cb_outline_size_changed (GtkAdjustment *adj, StylePrefState *state)
 
 	g_return_if_fail (style != NULL);
 
-	style->line.width = go_rint (adj->value * 100.) / 100.;
+	style->line.width = go_rint (gtk_adjustment_get_value (adj) * 100.) / 100.;
 	set_style (state);
 }
 
@@ -247,7 +247,7 @@ cb_line_size_changed (GtkAdjustment *adj, StylePrefState const *state)
 
 	g_return_if_fail (style != NULL);
 
-	style->line.width = go_rint (adj->value * 100.) / 100.;
+	style->line.width = go_rint (gtk_adjustment_get_value (adj) * 100.) / 100.;
 	set_style (state);
 }
 
@@ -721,7 +721,7 @@ cb_marker_fill_color_changed (GOSelector *selector,
 static void
 cb_marker_size_changed (GtkAdjustment *adj, StylePrefState *state)
 {
-	go_marker_set_size (state->style->marker.mark, adj->value);
+	go_marker_set_size (state->style->marker.mark, gtk_adjustment_get_value (adj));
 	set_style (state);
 }
 
@@ -2160,7 +2160,8 @@ go_style_set_cairo_line (GOStyle const *style, cairo_t *cr)
 	cairo_set_line_join (cr, style->line.join);
 	cairo_set_miter_limit (cr, style->line.miter_limit);
 	line_dash = go_line_dash_get_sequence (style->line.dash_type, width);
-	if (style->line.cap == CAIRO_LINE_CAP_BUTT && style->line.dash_type != GO_LINE_SOLID) {
+	if (style->line.cap == CAIRO_LINE_CAP_BUTT &&
+	    style->line.dash_type != GO_LINE_SOLID) {
 		unsigned i;
 		for (i = 0; i < line_dash->n_dash; i++)
 			if (line_dash->dash[i] == 0.)
